package depGui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Arc2D;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.QuadCurve2D;
import java.awt.image.BufferedImage;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class DibuixGraf<T> extends JPanel
{	
	private class Vertex {
		int pes;
		public T id; //Primary Key
		Ellipse2D dib;		
		int distance;
		boolean isDead;
		Vertex pare;
		TipusNode tipus;
		
		public boolean equals(Vertex v) {
			return v.id == id && v.pes == pes;
		}
		
		public String toString() {
			return id.toString();
		}
	}
	
	private class Aresta {
		Vertex from;
		Vertex to;
		boolean bidirecc;
		
		public Aresta(Vertex from, Vertex to) {
			this.from = from;
			this.to = to;
			bidirecc=false;
		}
		
		public boolean equals(Aresta a) {
			return (a.from.equals(from) && a.to.equals(to)) || (bidirecc && equalsInverse(a));
		}
		
		public boolean equalsInverse(Aresta a) {
			return a.from.equals(to) && a.to.equals(from);
		}
		
		public boolean fromEquals(Vertex from) {
			return this.from==from || (bidirecc && this.to==from);
		}
		
		public boolean toEquals(Vertex to) {
			return this.to==to || (bidirecc && this.from==to);
		}
		
		public String toString() {
			return "from :"+from.toString()+", to:"+to.toString()+", bidirecc: "+bidirecc;
		}
	}

	public enum Tipus { eliptic, circular, arbre }
	public enum TipusNode { normal, arrel, incorrecte, especial }
	
	private HashMap<T, Vertex> m_vertexs;
	private List<Aresta> m_arestes;
	private Vertex m_roots;
	private Vertex m_selected;
	private Tipus m_tipus;
	private boolean m_minim;
	private boolean m_rootShow;
	
	private static final long serialVersionUID = 1L;
	private static final Color fonsColor = Color.white;
	private static final Color fontColor = Color.black;
	private static final Color seleccioColor = Color.red;
	private static final Color seleccio2Color = Color.blue;
	private static final Color arestaNoSeleColor = new Color(135, 180, 180);
	private static final Color arestaColor = new Color(185, 230, 230);
	private static final Color nodeColor = new Color(255, 200, 0); //La 3a component depen del pes
	private static final Color incorrecteColor = new Color(255, 100, 0);
	private static final Color arrelColor = new Color(100, 255, 50);
	private static final Color especialColor = new Color(200, 255, 50);
	
	public Dimension getMinimumSize()
	{
		return new Dimension(200, 100);
	}
	
	public Dimension getPreferredSize()
	{
		return new Dimension(200, 100);
	}
	
	public void setRootShow(boolean b) { m_rootShow=b; } 
	
	public DibuixGraf()
	{
		clear();
		m_tipus = Tipus.circular;
		m_minim = false;
		m_rootShow = false;
		this.setBackground(fonsColor);
		
		MouseListener ml= new MouseListener()
		{
			public void mouseClicked(MouseEvent e) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent e) {
				if(e.getButton()==MouseEvent.BUTTON1)
					click((int) e.getX(), (int) e.getY());
				else
					setMinim(!minim());
			}
		};
		this.addMouseListener(ml);
		
	}
	
	public void addVertex(T id, int pes)
	{
		addVertex(id, pes, TipusNode.normal);
	}
	
	public void addVertex(T id, int pes, TipusNode t)
	{
		Vertex v = new Vertex();
		v.pes = pes;
		v.id = id;
		m_vertexs.put(id, v);
		v.tipus = t;
		
		if(TipusNode.arrel.equals(t))
			m_roots = v;
		
	}
	
	public boolean addAresta(T from, T to)
	{
		Aresta na = null;
		try {
			na = new Aresta(vertex(from), vertex(to));
		} catch(NoSuchElementException e) {
			return false;
		}
		boolean ex=false, bi=false;
		for(Aresta a : m_arestes) {
			if(na.equals(a)) {
				ex=true;
				break;
			} else if(na.equalsInverse(a)) {
				bi=true;
				a.bidirecc=true;
				break;
			}
		}
		
		if(!ex && !bi)
			m_arestes.add(na);
			
		return true;
	}
	
	private Vertex vertex(T id) throws NoSuchElementException
	{
		Vertex vr=m_vertexs.get(id);
		
		if(vr==null)
			throw new NoSuchElementException("No existeix "+ id.toString());
		
		return vr;
	}
	
	public boolean isRoot(T id)
	{
		return m_roots.id.equals(id);
	}
	
	private boolean isRoot(Vertex v)
	{
		return m_roots.equals(v);
	}
	
	//// Metodes de visualitzacio
	private void circular(Graphics2D g2)
	{
		
		int i=0, n=m_vertexs.size();
		double wx=g2.getClipBounds().getWidth()/2, wy=g2.getClipBounds().getHeight()/2;	//Coordenades centre
		double w, h;
		double fact=1., sfact=1.8;
		
		for(Vertex v : m_vertexs.values()) { //Situem els nodes
			if(isRoot(v))
				sfact=2.3;
			else
				sfact=1.8;
			
			w=sfact*g2.getClipBounds().getWidth()/n;
			h=sfact*g2.getClipBounds().getHeight()/n;
			
			w = Math.min(w, 80);
			w = Math.min(w, h);
			h = w;
			
			double dist = Math.min(wx, wy);
			double x = wx + (dist-30)*Math.sin(i*2*Math.PI/n)*fact -w/2;
			double y = wy + (dist-30)*Math.cos(i*2*Math.PI/n)*fact -h/2;
			v.dib = new Ellipse2D.Double(x, y, w, h);
			i++;
		}
	}
	
	private void eliptica(Graphics2D g2)
	{
		int i=0, n=m_vertexs.size();
		double wx=g2.getClipBounds().getWidth()/2, wy=g2.getClipBounds().getHeight()/2;	//Coordenades centre
		double w, h;
		double fact=.99, sfact=1.8;
		
		for(Vertex v : m_vertexs.values()) { //Situem els nodes
			if(isRoot(v))
				sfact=2.3;
			else
				sfact=1.8;
			w=sfact*g2.getClipBounds().getWidth()/n;
			h=sfact*g2.getClipBounds().getHeight()/n;
			
			w = Math.min(w, 80);
			h = Math.min(h, 80);
			
			double x = wx + (wx-30)*Math.sin(i*2*Math.PI/n)*fact -w/2;
			double y = wy + (wy-30)*Math.cos(i*2*Math.PI/n)*fact -h/2;
			v.dib = new Ellipse2D.Double(x, y, w, h);
			i++;
		}
	}
	
	private void dibuixaArestaArc(Graphics2D g2, Aresta a)
	{
		double wx=g2.getClipBounds().getWidth()/2, wy=g2.getClipBounds().getHeight()/2;
		double fx, fy, tx, ty;
		fx=a.from.dib.getCenterX();
		fy=a.from.dib.getCenterY();
		tx=a.to.dib.getCenterX();
		ty=a.to.dib.getCenterY();
		
		if(a.from==a.to) {
			Ellipse2D e;
			final double dist = Point2D.distance(fx, fy, wx, wy), mfact=0.5;
			double x, y, w, h;
			w = a.to.dib.getWidth()*mfact;
			h = a.to.dib.getHeight()*mfact;
			x = fx + w*(fx-wx)/dist;
			y = fy + h*(fy-wy)/dist;
			e = new Ellipse2D.Double(x-w/2, y-h/2, w, h);
			g2.draw(e);
			return;
		}
		
		Point2D p1 = new Point2D.Double(fx, fy);
		Point2D p2 = new Point2D.Double(wx, wy);
		Point2D p3 = new Point2D.Double(tx, ty);
		
		double th = angle(p1, p2, p3);
		double rad = p1.distance(p2);
		
		if(Math.abs(Math.tan(th/2)) < 10) { 
			double rad2 = rad*Math.tan(th/2);
			Arc2D.Double arc = new Arc2D.Double();
			arc.setArcByTangent(p1, p2, p3, rad2);
			g2.draw(arc);
			
			Arc2D.Double meitat = (Arc2D.Double) arc.clone();
			meitat.setAngleExtent(arc.getAngleExtent()/2);
			
			double px, py;
			px = p1.getX()-p3.getX();
			py = p1.getY()-p3.getY();
			
			dibuixaFletxa(g2, meitat.getEndPoint(), Math.atan2(py, px), a.bidirecc);
		} else
			dibuixaArestaQuad(g2, a, 0.9);
	}
	
	public static double angle(Point2D p1, Point2D p2, Point2D p3)
	{
		Point2D p = new Point2D.Double(p1.getX()-p2.getX(), p1.getY()-p2.getY()), pp = new Point2D.Double(p3.getX()-p2.getX(), p3.getY()-p2.getY());
		double cosi = (p.getX()*pp.getX() + p.getY()*pp.getY()) / (p.distance(0, 0)*pp.distance(0, 0));
		return Math.acos(cosi);
	}
	
	private void dibuixaVertex(Graphics2D g2, Vertex v) 
	{
		if(v.dib==null)
			return;
		
		if(!m_rootShow && v==m_roots)
			return;
		
		double x, y, wt, ht, w, h, sombra=-2;
		x=v.dib.getCenterX();
		y=v.dib.getCenterY();
		w=v.dib.getWidth();
		h=v.dib.getHeight();
		
		sombra = w<10 ? -w/5 : sombra;
		
		Font f = g2.getFont();
		f=f.deriveFont((float) 10);
		g2.setFont(f);
		
		if(v==m_selected)
			g2.setPaint(seleccioColor);
		else 
			g2.setPaint(Color.black);
		Ellipse2D vora = new Ellipse2D.Double(x-w/2+sombra, y-h/2+sombra, w, h);
		g2.fill(vora);
		
		switch(v.tipus) {
		case normal:
			g2.setPaint(nodeColor);
			break;
		case arrel:
			g2.setPaint(arrelColor);
			break;
		case incorrecte:
			g2.setPaint(incorrecteColor);
			break;
		case especial:
			g2.setPaint(especialColor);
			break;
		}
		g2.fill(v.dib);
		
		g2.setPaint(fontColor);
		wt=g2.getFontMetrics().stringWidth(v.id.toString());
		ht=g2.getFontMetrics().getHeight();
		
		x=x-wt/2;
		
		if(x<0)
			x=0;
		else if(x+wt>g2.getClipBounds().getWidth())
			x=g2.getClipBounds().getWidth()-wt;
		
		
		if(wt>2*w) {
			if(v==m_selected) {
				g2.drawString(v.id.toString(), (float) x, (float) (vora.getMaxY()+ht));
			} 
		} else {
			g2.drawString(v.id.toString(), (float) x, (float) (y+ht/4));
		}
	}
	
	private void dibuixaArestaQuad(Graphics2D g2, Aresta a, double fact)
	{
		double wx=getWidth()/2, wy=getHeight()/2;
		double fx, fy, tx, ty;
	
		fx=a.from.dib.getCenterX();
		fy=a.from.dib.getCenterY();
		tx=a.to.dib.getCenterX();
		ty=a.to.dib.getCenterY();
		
		if(a.from==a.to) {
			final double dist = Point2D.distance(fx, fy, wx, wy), mfact=0.5;
			double x, y, w, h;
			w = a.to.dib.getWidth()*mfact;
			h = a.to.dib.getHeight()*mfact;
			x = fx + w*(fx-wx)/dist;
			y = fy + h*(fy-wy)/dist;
			
			g2.draw(new Ellipse2D.Double(x-w/2, y-h/2, w, h));
			return;
		}
		
		double cx=fx+(tx-fx)/2, cy=fy+(ty-fy)/2;
		cx = (cx-wx)*fact+wx;
		cy = (cy-wy)*fact+wy;
        
		QuadCurve2D q = new QuadCurve2D.Double(fx, fy, cx, cy, tx, ty);
		g2.draw(q);
		
		QuadCurve2D l=new QuadCurve2D.Double(), r=null;
		q.subdivide(l, r);
		
		dibuixaFletxa(g2, new Point2D.Double(l.getX2(), l.getY2()), Math.atan2(q.getY1()-q.getY2(), q.getX1()-q.getX2()), a.bidirecc);
	}
	
	private void dibuixaArestaQuadV(Graphics2D g2, Aresta a, double fact)
	{
		double wx=g2.getClipBounds().getWidth()/2, wy=g2.getClipBounds().getHeight()/2;
		double fx, fy, tx, ty;
	
		fx=a.from.dib.getCenterX();
		fy=a.from.dib.getCenterY();
		tx=a.to.dib.getCenterX();
		ty=a.to.dib.getCenterY();
		
		if(a.from==a.to) {
			final double dist = Point2D.distance(fx, fy, wx, wy), mfact=0.5;
			double x, y, w, h;
			w = a.to.dib.getWidth()*mfact;
			h = a.to.dib.getHeight()*mfact;
			x = fx + w*(fx-wx)/dist;
			y = fy + h*(fy-wy)/dist;
			
			g2.draw(new Ellipse2D.Double(x-w/2, y-h/2, w, h));
			return;
		}
		
		double cx=fx+(tx-fx)/2, cy=fy+(ty-fy)/2;
		cy += g2.getClipBounds().getHeight()/4;
        
		QuadCurve2D q = new QuadCurve2D.Double(fx, fy, cx, cy, tx, ty);
		g2.draw(q);
		
		QuadCurve2D l=new QuadCurve2D.Double(), r=null;
		q.subdivide(l, r);
		
		dibuixaFletxa(g2, new Point2D.Double(l.getX2(), l.getY2()), Math.atan2(q.getY1()-q.getY2(), q.getX1()-q.getX2()), a.bidirecc);
	}
	
	private void dibuixaArestaRecta(Graphics2D g2, Aresta a)
	{
		double wx=g2.getClipBounds().getWidth()/2, wy=g2.getClipBounds().getHeight()/2;
		double fx, fy, tx, ty;
		
		fx=a.from.dib.getCenterX();
		fy=a.from.dib.getCenterY();
		tx=a.to.dib.getCenterX();
		ty=a.to.dib.getCenterY();
		
		if(a.from==a.to) {
			final double dist = Point2D.distance(fx, fy, wx, wy), mfact=0.5;
			double x, y, w, h;
			w = a.to.dib.getWidth()*mfact;
			h = a.to.dib.getHeight()*mfact;
			x = fx + w*(fx-wx)/dist;
			y = fy + h*(fy-wy)/dist;
			
			g2.draw(new Ellipse2D.Double(x-w/2, y-h/2, w, h));
			return;
		}
		
		double cx=fx+(tx-fx)/2, cy=fy+(ty-fy)/2;
        
		Line2D.Double q = new Line2D.Double(fx, fy, tx, ty);
		g2.draw(q);
		
		dibuixaFletxa(g2, new Point2D.Double(cx, cy), Math.atan2(q.getY1()-q.getY2(), q.getX1()-q.getX2()), a.bidirecc);
	}
	
	private void dibuixaFletxa(Graphics2D g2, Point2D p, double deg, boolean bidirec)
	{
		Point2D up, down;
		final double factx=8, facty=3;
		

		up = new Point2D.Double(factx, facty);
		down = new Point2D.Double(factx, -facty);
		
		up.setLocation(up.getX()*Math.cos(deg)-up.getY()*Math.sin(deg), up.getX()*Math.sin(deg)+up.getY()*Math.cos(deg));
		down.setLocation(down.getX()*Math.cos(deg)-down.getY()*Math.sin(deg), down.getX()*Math.sin(deg)+down.getY()*Math.cos(deg));

		up.setLocation(p.getX()+up.getX(), p.getY()+up.getY());
		down.setLocation(p.getX()+down.getX(), p.getY()+down.getY());
		
		g2.draw(new Line2D.Double(p, up));
		g2.draw(new Line2D.Double(p, down));
		
		if(bidirec)
			dibuixaFletxa(g2, p, deg+Math.PI, false);
	}
	
	private void click(double x, double y)
	{
		m_selected=null;
		Point2D p = new Point2D.Double(x, y);
		for(Vertex v : m_vertexs.values()) {
			if(v.dib!=null && v.dib.contains(p))
				m_selected = v;
		}
		
		if(m_selected==null)
			System.out.println("agua");
		else
			System.out.println(x+" - "+y+": "+m_selected.toString());
		
		this.repaint();
	}
	
	public boolean hasVertex(T id)
	{
		return m_vertexs.containsKey(id);
	}
	
	public void paint(Graphics g) {
		g.setClip(0, 0, this.getWidth(), this.getHeight());
		paint(g, true);
	}
	
	public void paint(Graphics g, boolean gui)
	{
		if(gui)
			super.paint(g);
		Graphics2D g2 = (Graphics2D) g;
		g2.addRenderingHints(new RenderingHints(
				RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
		g2.addRenderingHints(new RenderingHints(
				RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON));
		
		if(m_vertexs.isEmpty())
			return;
		
		switch(m_tipus) {
			case circular:
				circular(g2);
				break;
			case eliptic:
				eliptica(g2);
				break;
			case arbre:
				arbre(g2);
				break;
		}
		
		List<Aresta> selected = new LinkedList<Aresta>();
		
		for(Aresta a : m_arestes) {
			if(m_selected!=a.from && m_selected!=a.to)
				dibuixaAresta(g2, a);
			else
				selected.add(a);
		}
		
		for(Aresta a : selected)
			dibuixaAresta(g2, a);
		
		for(Vertex v : m_vertexs.values())
			dibuixaVertex(g2, v);
	}
	
	private int amplada(Vertex v) {
		int amp=1;
		Set<Aresta> as = arestes(v);
		for(Aresta a : as) {
			if(m_minim && a.from==a.to.pare)
				amp += amplada(a.to);
		}
		return amp;
	}
	
	Set<Vertex> countPis(int dist) {
		Set<Vertex> c= new LinkedHashSet<Vertex>();
		for(Vertex v : m_vertexs.values()) {
			if(v.distance==dist)
				c.add(v);
		}
		return c;
	}
	
	private void arbre(Graphics2D g2)
	{
		int maxdis=0;
		for(Vertex v : m_vertexs.values()) {
			if(v.distance<INFINITY && maxdis<v.distance)
				maxdis=v.distance;
		}
		if(maxdis==1)
			return;
		
		int pis;
		final double hpis = ((int) g2.getClipBounds().getHeight())/(m_rootShow ? maxdis+1 : maxdis);
		final double w=40, h=40;
		List<Vertex> vpis_ant = null, vpis_act;
		for(pis= m_rootShow ? 0 : 1; pis<=maxdis; pis++) {
			Set<Vertex> vpis = countPis(pis);
			double hp = hpis*(m_rootShow ? pis : pis-1);
			
			final double wpis = g2.getClipBounds().getWidth()/ (double) vpis.size();
			final double wnod = wpis > w ? w : wpis;
			
			int i=0;
			vpis_act = new ArrayList<Vertex>(vpis.size());
			if(vpis_ant==null) {
				for(Vertex v : vpis) {
					v.dib = new Ellipse2D.Double(wpis*i+wpis/2-w/2, hp+h/2, w, h);
					
					i++;
					vpis_act.add(v);
				}
			} else {
				for(Vertex v : vpis_ant) {
					for(Vertex v1 : vpis) {
						if(v1.pare==v) {
							v1.dib = new Ellipse2D.Double(wpis*i+wpis/2-wnod/2, hp+h/2, wnod, wnod);
							i++;
							vpis_act.add(v1);
						}
					}
				}
			}
			vpis_ant = vpis_act;
		}
	}
	
	
	private void dibuixaAresta(Graphics2D g2, Aresta a)
	{
		if(!m_rootShow && (a.from == m_roots || a.to == m_roots))
			return;
		
		if(a.from.dib==null || a.to.dib==null) {
			System.out.println("error>> "+a.toString());
			return;
		}
		
		Aresta j = a;
		if(j.bidirecc && j.from!=j.to.pare) {
			Vertex v=j.to;
			j.to = j.from;
			j.from = v;
		}
		
		if(j.fromEquals(m_selected))
			g2.setPaint(seleccioColor);
		else if(j.toEquals(m_selected))
			g2.setPaint(seleccio2Color);
		else if(m_selected==null)
			g2.setPaint(arestaNoSeleColor);
		else
			g2.setPaint(arestaColor);
		
		if(m_minim && j.from!=j.to.pare)
			return;
		switch(m_tipus) {
		case circular:
			dibuixaArestaArc(g2, j);
			break;
		case eliptic:
			dibuixaArestaQuad(g2, j, 0.1);
			break;
		case arbre:
			if(j.from.distance == j.to.distance)
				dibuixaArestaQuadV(g2, j, -0.3);
			else
				dibuixaArestaRecta(g2, j);
			break;
		}
	}
	
	public T selected()
	{
		return m_selected.id;
	}
	
	public void setTipus(Tipus t)
	{
		m_tipus = t;
		repaint();
	}
	
	public void camiMinim()
	{
		dijkstra(m_roots);
	}
	
	final private int INFINITY = 32000;
	
	private void dijkstra(Vertex source) {
			
			for(Vertex i : m_vertexs.values()) {
				if(i == source) {
					i.distance = 0;
					i.isDead = false;
			} else {
				i.distance = INFINITY;
				i.isDead = false;
			}
		}
		
		Iterator i = m_vertexs.values().iterator();
		while(i.hasNext()) {
			i.next();
			Vertex next=null;
			int min = INFINITY+1;
			for(Vertex j : m_vertexs.values()) {
				if(!j.isDead && j.distance < min) {
					next = j;
					min = j.distance;
				}
			}
			
			Set<Aresta> as = arestes(next);
			for(Aresta j : as) {
				/*if(j.to.distance > next.distance + 1)
				{
					j.to.distance = next.distance +1; // + j.weight;
					j.to.pare = next;
				}*/
				Aresta a = j;
				if(j.bidirecc && j.from!=next) {
					Vertex v=j.to;
					j.to = j.from;
					j.from = v;
				}
				
				if(a.to.distance > next.distance + 1)
				{
					a.to.distance = next.distance +1; // + j.weight;
					a.to.pare = next;
				}
			}
			next.isDead = true;
		}
		
		/*for(Vertex j : m_vertexs){
			System.out.println("La distancia entre el node "+source+" i "+j+" es "+ j.distance);
		}*/
	}
	
	private Set<Aresta> arestes(Vertex v)
	{
		Set<Aresta> as = new LinkedHashSet<Aresta>();
		for(Aresta a : m_arestes) {
			if(a.fromEquals(v))
				as.add(a);
		}
		return as;
	}
	
	public void setMinim(boolean m)
	{
		m_minim = m;
		repaint();
	}
	
	public boolean minim()
	{
		return m_minim;
	}
	
	public Aresta link(Vertex from, Vertex to)
	{
		Aresta r = null;
		for(Aresta a : m_arestes) {
			if((a.from==from && a.to==to) || (a.bidirecc && a.from==to && a.to==from)) {
				r=a;
				break;
			}
		}
		return r;
	}
	
	public boolean export(String path, String format, String color) {
		return export(path, format, Color.getColor(color));
	}
	
	//Support jpeg, png , bmp
	public boolean export(String path, String format, Color color)
	{
		final int wi=2000, he=2000; //TODO: You could make me dynamic, don't you?
		BufferedImage imatge = new BufferedImage(wi, he,BufferedImage.TYPE_INT_RGB);
		Graphics2D h = (Graphics2D) imatge.getGraphics();
		h.setClip(0, 0, wi, he);
		
		Color c = h.getBackground();
		h.setBackground(color);
		this.paint(h, false);
		
			FileOutputStream fd;
			try {
				fd = new FileOutputStream(path);
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				return false;
			}
			try {
				ImageIO.write(imatge, format, fd);
				fd.close();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
			  
		h.setBackground(c);
		
		return true;
	}
	
	public boolean exportHtml(String path, String bgcolor, boolean b)
	{
		FileOutputStream fd=null;
		try {
			fd = new FileOutputStream(path);
		} catch (FileNotFoundException e) {
			//e.printStackTrace();
			return false;
		}
		PrintStream escriu = new PrintStream(fd);
		escriu.println("<html>\n<head>\n\t<title>"+m_roots.id+"\n\t</title>\n</head>\n<body bgcolor=\""+bgcolor+"\"><pre>");
		escriuHtml(escriu, m_roots, -1, b);
		escriu.println("</pre></body></html>");
		
		return true;
	}
	
	private void escriuHtml(PrintStream escriu, Vertex v, int indent, boolean b)
	{
		Set<Aresta> as = arestes(v);
		for(int i=0; i<indent; i++) {
			escriu.print("\t");
		}
		int m=arestes(v).size()/2+4;
		if(v!=m_roots)
			if(b)
				escriu.println("<a href='"+v.id+"'><font size='"+m+"'>"+v.id+"</font></a>");
			else
				escriu.println("<a href='"+v.id+"'>"+v.id+"</a>");
		for(Aresta a : as) {
			if(a.to.pare==v)
				escriuHtml(escriu, a.to, indent+1, b);
			else if (a.bidirecc && a.from.pare==v) {
				escriuHtml(escriu, a.from, indent+1, b);
			}
		}
	}
	
	public boolean hasSeleccio() {
		return m_selected!=null;
	}
	
	public void clear() {
		m_vertexs = new HashMap<T, Vertex>();
		m_arestes = new LinkedList<Aresta>();
		m_roots = null;
		m_selected = null;
	}
	
	public Tipus tipus() {
		return m_tipus;
	}
}
